/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app_i2c.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app_i2c.h"
//$ ?????header file
#include "app_ur.h"
#include "driver/i2c/drv_i2c.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

#define MCP9800_ADDRESS 0x4D
#define MCP9800_REG_TA  0x0

DRV_HANDLE I2C1Handle_9800;
DRV_I2C_TRANSFER_HANDLE I2C1WriteReadHandle_9800;
volatile int I2C1Complete_9800 = false;
char I2C1WriteBuf_9800[20];
char I2C1ReadBuf_9800[20];

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_I2C_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_I2C_DATA app_i2cData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/

void I2C1TransferHandler_9800( DRV_I2C_TRANSFER_EVENT event,
                               DRV_I2C_TRANSFER_HANDLE transferHandle,
                               uintptr_t context )
{
    switch( event )
    {
    case DRV_I2C_TRANSFER_EVENT_PENDING:        break;
    case DRV_I2C_TRANSFER_EVENT_COMPLETE:
        I2C1Complete_9800 = true;
        break;
    case DRV_I2C_TRANSFER_EVENT_HANDLE_EXPIRED:
    case DRV_I2C_TRANSFER_EVENT_ERROR:
    case DRV_I2C_TRANSFER_EVENT_HANDLE_INVALID:
        break;
    }
}



// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


/* TODO:  Add any necessary local functions.
*/


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_I2C_Initialize ( void )

  Remarks:
    See prototype in app_i2c.h.
 */

void APP_I2C_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    app_i2cData.state = APP_I2C_STATE_INIT;



    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}


/******************************************************************************
  Function:
    void APP_I2C_Tasks ( void )

  Remarks:
    See prototype in app_i2c.h.
 */

void APP_I2C_Tasks ( void )
{

    /* Check the application's current state. */
    switch ( app_i2cData.state )
    {
        /* Application's initial state. */
        case APP_I2C_STATE_INIT:
        {
            bool appInitialized = true;

            //$ open driver -> ?? CallBack function
            //$ ??code????, ????error checking
            I2C1Handle_9800 = DRV_I2C_Open(DRV_I2C_INDEX_0, DRV_IO_INTENT_READWRITE);
            DRV_I2C_TransferEventHandlerSet(I2C1Handle_9800, I2C1TransferHandler_9800, 0);
            
            
            if (appInitialized)
            {

                app_i2cData.state = APP_I2C_STATE_SERVICE_TASKS;
            }
            break;
        }

        case APP_I2C_STATE_SERVICE_TASKS:
        {

            break;
        }

        /* TODO: implement your application state machine.*/
        //$ Requst I2C Read/Write
        case APP_I2C_STATE_RW_REQUEST:
        {
            I2C1WriteBuf_9800[0] = MCP9800_REG_TA;
            DRV_I2C_WriteReadTransferAdd( I2C1Handle_9800, MCP9800_ADDRESS,
                    I2C1WriteBuf_9800, 1, I2C1ReadBuf_9800, 2, &I2C1WriteReadHandle_9800 );
            app_i2cData.state=APP_I2C_STATE_RW_WAIT_FOR_COMPLETED;
            break;
        }
        
        

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}


/*******************************************************************************
 End of File
 */
