/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app_ur.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app_ur.h"
#include "driver/usart/drv_usart.h"
#include "app_i2c.h"
#include "stdio.h"
#include "string.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

DRV_HANDLE  Uart1Handle;  // ??open driver?return?handle
DRV_USART_BUFFER_HANDLE Uart1WriteTransferHandle;  // write request?handle
volatile int URWriteComplete = false;   // state machine ??flag
char Uart1WriteBuf[50];
APP_UR_DATA app_ur_Data;   // app client ?state machine maagement

extern volatile int I2C1Complete_9800;
extern char I2C1ReadBuf_9800[20];
extern APP_I2C_DATA app_i2cData;

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_UR_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_UR_DATA app_urData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/
void Uart1TransferHandler( DRV_USART_BUFFER_EVENT event, DRV_USART_BUFFER_HANDLE bufferHandle, uintptr_t context )
{
    switch( event )
    {
    case DRV_USART_BUFFER_EVENT_PENDING: break;
    case DRV_USART_BUFFER_EVENT_COMPLETE:
        if(bufferHandle == Uart1WriteTransferHandle)
            URWriteComplete = true;
        break;
    case DRV_USART_BUFFER_EVENT_HANDLE_EXPIRED:
    case DRV_USART_BUFFER_EVENT_ERROR:
    case DRV_USART_BUFFER_EVENT_HANDLE_INVALID:
        break;
    }
}



// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


/* TODO:  Add any necessary local functions.
*/


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_UR_Initialize ( void )

  Remarks:
    See prototype in app_ur.h.
 */

void APP_UR_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    app_urData.state = APP_UR_STATE_INIT;



    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}


/******************************************************************************
  Function:
    void APP_UR_Tasks ( void )

  Remarks:
    See prototype in app_ur.h.
 */

void APP_UR_Tasks ( void )
{

    /* Check the application's current state. */
    switch ( app_urData.state )
    {
        /* Application's initial state. */
        case APP_UR_STATE_INIT:
        {
            bool appInitialized = true;
            
            //$ open driver -> ?? CallBack function
            //$ ??code????, ????error checking
            Uart1Handle = DRV_USART_Open(DRV_USART_INDEX_0, DRV_IO_INTENT_READWRITE);
            DRV_USART_BufferEventHandlerSet(Uart1Handle, Uart1TransferHandler, 0);
            //Uart1WriteBuf[3]='\n';
            //Uart1WriteBuf[4]='\r';

            if (appInitialized)
            {

                app_urData.state = APP_UR_STATE_SERVICE_TASKS;
            }
            break;
        }

        case APP_UR_STATE_SERVICE_TASKS:
        {

            break;
        }

        /* TODO: implement your application state machine.*/
        //$$ Requst UR Write
        case APP_UR_STATE_SENT_REQUEST:
        {
            if (I2C1Complete_9800 == true) 
            {
                sprintf(Uart1WriteBuf, "Temp = %2.1f\n\r", 
                        (float)((I2C1ReadBuf_9800[0]<<4)|(I2C1ReadBuf_9800[1]>>4))/16.0f);                
                DRV_USART_WriteBufferAdd(Uart1Handle, Uart1WriteBuf, strlen(Uart1WriteBuf), &Uart1WriteTransferHandle);
                app_urData.state = APP_UR_STATE_SENT_WAIT_FOR_COMPLETED;
                I2C1Complete_9800=false;
            }
            break;
        }
        case APP_UR_STATE_SENT_WAIT_FOR_COMPLETED:
        {
            break;
        }
            
        case APP_UR_STATE_SENT_COMPLETED:
        {
            
            break;
        }

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}


/*******************************************************************************
 End of File
 */
